/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;

/**
 * AlarmReceiver 类接收并处理广播消息。
 * 当接收到某个广播时，它会启动 AlarmAlertActivity。
 */
public class AlarmReceiver extends BroadcastReceiver {

    /**
     * 当接收到广播时，执行该方法。
     * 该方法会启动 AlarmAlertActivity，显示提醒内容。
     *
     * @param context 当前的上下文对象
     * @param intent  接收到的广播消息意图
     */
    @Override
    public void onReceive(Context context, Intent intent) {
        // 设置 intent 的目标活动为 AlarmAlertActivity
        intent.setClass(context, AlarmAlertActivity.class);
        
        // 添加 FLAG 来启动新的任务栈，这样 AlarmAlertActivity 会作为一个新的活动启动
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        
        // 启动 AlarmAlertActivity
        context.startActivity(intent);
    }
}
