/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.net.Uri;

public class Notes {
    // 认证标识符，用于在内容提供者中区分不同的数据集合
    public static final String AUTHORITY = "micode_notes";

    // 用于日志记录的标签
    public static final String TAG = "Notes";

    // 用于表示不同类型的文件
    public static final int TYPE_NOTE = 0;
    public static final int TYPE_FOLDER = 1;
    public static final int TYPE_SYSTEM = 2;

    /**
     * 以下ID是系统文件夹的标识符
     * {@link Notes#ID_ROOT_FOLDER} 表示默认文件夹
     * {@link Notes#ID_TEMPARAY_FOLDER} 表示临时文件夹
     * {@link Notes#ID_CALL_RECORD_FOLDER} 表示用于存储通话记录的文件夹
     */
    public static final int ID_ROOT_FOLDER = 0;
    public static final int ID_TEMPARAY_FOLDER = -1;
    public static final int ID_CALL_RECORD_FOLDER = -2;
    public static final int ID_TRASH_FOLER = -3;

    // Intent 额外数据的键，用于传递提醒日期、背景颜色、Widget ID、文件夹ID等信息
    public static final String INTENT_EXTRA_ALERT_DATE = "net.micode.notes.alert_date";
    public static final String INTENT_EXTRA_BACKGROUND_ID = "net.micode.notes.background_color_id";
    public static final String INTENT_EXTRA_WIDGET_ID = "net.micode.notes.widget_id";
    public static final String INTENT_EXTRA_WIDGET_TYPE = "net.micode.notes.widget_type";
    public static final String INTENT_EXTRA_FOLDER_ID = "net.micode.notes.folder_id";
    public static final String INTENT_EXTRA_CALL_DATE = "net.micode.notes.call_date";

    // Widget 类型常量
    public static final int TYPE_WIDGET_INVALIDE = -1;
    public static final int TYPE_WIDGET_2X = 0;
    public static final int TYPE_WIDGET_4X = 1;

    public static class DataConstants {
        // 文本便签的 MIME 类型
        public static final String NOTE = TextNote.CONTENT_ITEM_TYPE;
        // 通话记录便签的 MIME 类型
        public static final String CALL_NOTE = CallNote.CONTENT_ITEM_TYPE;
    }

    /**
     * Uri 用于查询所有的便签和文件夹
     */
    public static final Uri CONTENT_NOTE_URI = Uri.parse("content://" + AUTHORITY + "/note");

    /**
     * Uri 用于查询具体数据
     */
    public static final Uri CONTENT_DATA_URI = Uri.parse("content://" + AUTHORITY + "/data");

    /**
     * NoteColumns 接口定义了便签和文件夹表中的列
     */
    public interface NoteColumns {
        // 唯一标识符
        public static final String ID = "_id";

        // 父文件夹的 ID
        public static final String PARENT_ID = "parent_id";

        // 创建日期
        public static final String CREATED_DATE = "created_date";

        // 最新修改日期
        public static final String MODIFIED_DATE = "modified_date";

        // 提醒日期
        public static final String ALERTED_DATE = "alert_date";

        // 文件夹名称或便签内容的摘要
        public static final String SNIPPET = "snippet";

        // 便签对应的 Widget ID
        public static final String WIDGET_ID = "widget_id";

        // 便签对应的 Widget 类型
        public static final String WIDGET_TYPE = "widget_type";

        // 便签的背景颜色 ID
        public static final String BG_COLOR_ID = "bg_color_id";

        // 是否有附件，1 表示有，0 表示无
        public static final String HAS_ATTACHMENT = "has_attachment";

        // 文件夹中的便签数量
        public static final String NOTES_COUNT = "notes_count";

        // 文件类型：文件夹或便签
        public static final String TYPE = "type";

        // 最近同步的 ID
        public static final String SYNC_ID = "sync_id";

        // 本地修改标记
        public static final String LOCAL_MODIFIED = "local_modified";

        // 移动到临时文件夹之前的原父文件夹 ID
        public static final String ORIGIN_PARENT_ID = "origin_parent_id";

        // Gtask 的 ID
        public static final String GTASK_ID = "gtask_id";

        // 版本号
        public static final String VERSION = "version";
    }

    /**
     * DataColumns 接口定义了数据表中的列
     */
    public interface DataColumns {
        // 唯一标识符
        public static final String ID = "_id";

        // MIME 类型
        public static final String MIME_TYPE = "mime_type";

        // 关联的便签 ID
        public static final String NOTE_ID = "note_id";

        // 创建日期
        public static final String CREATED_DATE = "created_date";

        // 最新修改日期
        public static final String MODIFIED_DATE = "modified_date";

        // 数据内容
        public static final String CONTENT = "content";

        // 通用数据列，用于存储整数类型数据
        public static final String DATA1 = "data1";

        // 通用数据列，用于存储整数类型数据
        public static final String DATA2 = "data2";

        // 通用数据列，用于存储文本类型数据
        public static final String DATA3 = "data3";

        // 通用数据列，用于存储文本类型数据
        public static final String DATA4 = "data4";

        // 通用数据列，用于存储文本类型数据
        public static final String DATA5 = "data5";
    }

    /**
     * TextNote 类继承自 DataColumns，专门处理文本便签相关的数据
     */
    public static final class TextNote implements DataColumns {
        // 模式，表示便签是否处于检查列表模式，1 为检查列表模式，0 为普通模式
        public static final String MODE = DATA1;

        public static final int MODE_CHECK_LIST = 1;

        // MIME 类型，用于表示文本便签的集合
        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/text_note";

        // MIME 类型，用于表示单个文本便签
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/text_note";

        // Uri 用于查询文本便签
        public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY + "/text_note");
    }

    /**
     * CallNote 类继承自 DataColumns，专门处理通话记录便签相关的数据
     */
    public static final class CallNote implements DataColumns {
        // 通话日期
        public static final String CALL_DATE = DATA1;

        // 通话的电话号码
        public static final String PHONE_NUMBER = DATA3;

        // MIME 类型，用于表示通话记录便签的集合
        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/call_note";

        // MIME 类型，用于表示单个通话记录便签
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/call_note";

        // Uri 用于查询通话记录便签
        public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY + "/call_note");
    }
}
