/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.exception;

/**
 * ActionFailureException 类是用于表示操作失败时抛出的异常。
 * 它继承自 RuntimeException，因此它是一个非检查型异常（unchecked exception），
 * 这意味着在方法签名中不需要声明此异常可能会被抛出。
 */
public class ActionFailureException extends RuntimeException {

    // 序列化版本唯一标识符，用于确保类的不同版本之间的兼容性。
    private static final long serialVersionUID = 4425249765923293627L;

    /**
     * 构造一个没有详细信息消息和原因的 ActionFailureException。
     */
    public ActionFailureException() {
        super();
    }

    /**
     * 使用指定的详细信息消息构造一个新的 ActionFailureException。
     * @param paramString 提供更多关于异常的信息的消息字符串。
     */
    public ActionFailureException(String paramString) {
        super(paramString);
    }

    /**
     * 使用指定的详细信息消息和原因构造一个新的 ActionFailureException。
     * @param paramString 提供更多关于异常的信息的消息字符串。
     * @param paramThrowable 导致当前异常的底层原因。
     */
    public ActionFailureException(String paramString, Throwable paramThrowable) {
        super(paramString, paramThrowable);
    }
}